#ifndef STARSInterface_h
#define STARSInterface_h

#include <cstdio>

class STARS_Program_t;
class STARS_Function_t;

#include "interfaces/STARSTypes.h"
#include <interfaces/abstract/STARSSegment.h>
#include <interfaces/abstract/STARSInstruction.h>
#include <interfaces/abstract/STARSInstructionID.h>

class STARS_Interface_t
{
	public:
		virtual ~STARS_Interface_t() { };
/*
 * Segment methods
 */

		/* return segment for virtual addr */
		virtual STARS_Segment_t *getseg(const STARS_ea_t &addr) = 0; 

		/* Return segment at index "index" */
		virtual STARS_Segment_t *getnseg(const int &index) = 0; 

		/* return number of segments */
		virtual std::size_t get_segm_qty() = 0;

		virtual STARS_Segment_t *get_next_seg(const STARS_ea_t &addr) = 0;

/*
 * Function methods
 */
		/* find out how many functions there are */
		virtual std::size_t get_func_qty() = 0;

		/* get the index-th function */
		virtual STARS_Function_t *getn_func(int index) = 0;

		/* get the function at the given address */
		virtual STARS_Function_t *get_func(STARS_ea_t index) = 0;

		/* get the function's paternal grandmother's maiden name. */
		virtual void get_func_name(const STARS_ea_t &ea, char* name, const std::size_t &len) = 0;

		// Instruction creation.
		virtual STARS_Instruction_t *CreateInst(STARS_InstructionID_t InstID) = 0;

		// IDA Pro does not permit direct usage of common file and string library calls, so we
		//  have to do them differently in the IDA Pro and IRDB interfaces.

		// File methods.
		virtual FILE *STARS_fopen(const char *file, const char *mode) = 0;
		virtual int STARS_fclose(FILE *fp) = 0;
		virtual int STARS_msg(const char *format, ...) = 0;
		virtual int STARS_fprintf(FILE *fp, const char *format, ...) = 0;
		virtual int STARS_fscanf(FILE *fp, const char *format, ...) = 0;
		virtual long STARS_ftell(FILE *fp) = 0;
		virtual int STARS_fgetc(FILE *fp) = 0;
		virtual char *STARS_fgets(char *buffer, int buflen, FILE *fp) = 0;
		virtual int STARS_fseek(FILE *fp, long offset, int whence) = 0;

		// String methods
		virtual char *STARS_strncat(char *dst, const char *src, std::size_t dstsize) = 0;
		virtual char *STARS_strncpy(char *dst, const char *src, std::size_t dstsize) = 0;
		virtual int STARS_snprintf(char *buffer, std::size_t n, const char *format, ...) = 0;

		// Disasm methods.
		virtual STARS_ssize_t STARS_tag_remove(char *instr, char *buf, std::size_t bufsize) = 0;
		virtual bool STARS_generate_disasm_line(STARS_ea_t addr, char *buf,	std::size_t bufsize, int flags = 0) = 0;

		// Misc. methods
		virtual bool STARS_getenv(const char *varname) const = 0;
		virtual bool VerboseLoopsMode(void) const { return this->STARS_getenv("STARS_VERBOSE_LOOPS"); };
		virtual void SetCGCBinary(void) = 0;
		virtual bool IsCGCBinary(void) const = 0;
		virtual bool InstHasNoCodeXrefs(STARS_InstructionID_t InstID) const = 0;
		virtual bool IsInstJumpTarget(STARS_InstructionID_t InstID) const = 0;
		virtual STARS_InstructionID_t FindFirstCallTarget(STARS_InstructionID_t CallInstID) const = 0; // Find call target; first one, if any, for indirect call

		/* IDA-only auditing */
		virtual void AuditTailChunkOwnership(void) = 0;
		virtual void AuditCodeTargets(void) = 0;

		virtual bool STARS_patch_byte(STARS_ea_t InstAddr, uint32_t ByteValue) = 0; // Patch IDA Pro database.

		virtual void Dump() { }; 



};

extern STARS_Interface_t* global_stars_interface;
extern STARS_Program_t *global_STARS_program;

#endif



